@@ -165,34 +165,46 @@ impl Fold for CommonJs {
 
         // Make a preliminary pass through to collect exported names ahead of time
         for item in &items {
-            if let ModuleItem::ModuleDecl(ModuleDecl::ExportNamed(NamedExport {
-                ref specifiers,
-                ..
-            })) = item
-            {
-                for ExportNamedSpecifier { orig, exported, .. } in
-                    specifiers.iter().filter_map(|e| match e {
-                        ExportSpecifier::Named(e) => Some(e),
-                        _ => None,
-                    })
-                {
-                    let exported = match &exported {
-                        Some(ModuleExportName::Ident(ident)) => Some(ident),
-                        Some(ModuleExportName::Str(..)) => {
-                            unimplemented!("module string names unimplemented")
+            match item {
+                ModuleItem::ModuleDecl(ModuleDecl::ExportNamed(NamedExport {
+                    ref specifiers,
+                    ..
+                })) => {
+                    for ExportNamedSpecifier { orig, exported, .. } in
+                        specifiers.iter().filter_map(|e| match e {
+                            ExportSpecifier::Named(e) => Some(e),
+                            _ => None,
+                        })
+                    {
+                        let exported = match &exported {
+                            Some(ModuleExportName::Ident(ident)) => Some(ident),
+                            Some(ModuleExportName::Str(..)) => {
+                                unimplemented!("module string names unimplemented")
+                            }
+                            _ => None,
+                        };
+                        let orig = match &orig {
+                            ModuleExportName::Ident(ident) => ident,
+                            _ => unimplemented!("module string names unimplemented"),
+                        };
+                        if let Some(exported) = &exported {
+                            exports.push(exported.sym.clone());
+                        } else {
+                            exports.push(orig.sym.clone());
                         }
-                        _ => None,
-                    };
-                    let orig = match &orig {
-                        ModuleExportName::Ident(ident) => ident,
-                        _ => unimplemented!("module string names unimplemented"),
-                    };
-                    if let Some(exported) = &exported {
-                        exports.push(exported.sym.clone());
-                    } else {
-                        exports.push(orig.sym.clone());
                     }
                 }
+                ModuleItem::ModuleDecl(ModuleDecl::ExportDecl(ExportDecl { decl, .. })) => {
+                    let mut found: Vec<Ident> = vec![];
+
+                    let mut v = DestructuringFinder { found: &mut found };
+                    decl.visit_with(&mut v);
+
+                    for ident in found {
+                        exports.push(ident.sym.clone());
+                    }
+                }
+                _ => {}
             }
         }
 
@@ -242,7 +254,6 @@ impl Fold for CommonJs {
                             init_export!(js_word!("default"))
                         }};
                         ($name:expr) => {{
-                            exports.push($name.clone());
                             initialized.insert($name.clone());
                         }};
                     }
@@ -784,16 +795,16 @@ impl Fold for CommonJs {
         let scope = &mut *scope_ref_mut;
 
         let scope = &mut *scope;
-        for (src, import) in scope.imports.drain(..) {
+        for (src, (src_span, import)) in scope.imports.drain(..) {
             let lazy = if scope.lazy_blacklist.contains(&src) {
                 false
             } else {
                 self.config.lazy.is_lazy(&src)
             };
 
-            let require = self
-                .resolver
-                .make_require_call(self.top_level_mark, src.clone());
+            let require =
+                self.resolver
+                    .make_require_call(self.top_level_mark, src.clone(), src_span);
 
             match import {
                 Some(import) => {
@@ -915,7 +926,57 @@ impl Fold for CommonJs {
         stmts
     }
 
-    fn fold_expr(&mut self, expr: Expr) -> Expr {
+    fn fold_expr(&mut self, mut expr: Expr) -> Expr {
+        if !self.config.preserve_import_meta {
+            // https://github.com/swc-project/swc/issues/1202
+            if let Expr::Member(MemberExpr {
+                obj,
+                prop: MemberProp::Ident(prop),
+                ..
+            }) = &mut expr
+            {
+                if &*prop.sym == "url" {
+                    if let Expr::MetaProp(MetaPropExpr {
+                        span,
+                        kind: MetaPropKind::ImportMeta,
+                        ..
+                    }) = &**obj
+                    {
+                        // require('url').pathToFileURL(__filename).toString()
+
+                        let url_module = CallExpr {
+                            span: DUMMY_SP,
+                            callee: quote_ident!("require").as_callee(),
+                            args: vec!["url".as_arg()],
+                            type_args: Default::default(),
+                        };
+
+                        let url_obj = CallExpr {
+                            span: DUMMY_SP,
+                            callee: url_module
+                                .make_member(quote_ident!("pathToFileURL"))
+                                .as_callee(),
+                            args: vec![Ident::new(
+                                "__filename".into(),
+                                DUMMY_SP.with_ctxt(
+                                    SyntaxContext::empty().apply_mark(self.top_level_mark),
+                                ),
+                            )
+                            .as_arg()],
+                            type_args: Default::default(),
+                        };
+
+                        *obj = Box::new(Expr::Call(CallExpr {
+                            span: *span,
+                            callee: url_obj.make_member(quote_ident!("toString")).as_callee(),
+                            args: Default::default(),
+                            type_args: Default::default(),
+                        }));
+                    }
+                }
+            }
+        }
+
         let top_level = self.in_top_level;
         Scope::fold_expr(self, quote_ident!("exports"), top_level, expr)
     }
